@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py,v 1.93 2006/11/13 22:33:00 richb Exp $"
-__version__   = "$Revision: 1.93 $"
-__date__      = "$Date: 2006/11/13 22:33:00 $"
+__id__        = "$Id: focus_tracking_presenter.py 1825 2006-12-13 22:35:06Z wwalker $"
+__version__   = "$Revision: 1825 $"
+__date__      = "$Date: 2006-12-13 17:35:06 -0500 (Wed, 13 Dec 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -391,7 +391,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 return
         except:
             pass
-        
+
         # Reclaim (delete) any scripts when desktop children go away.
         # The idea here is that a desktop child is an app. We also
         # generally do not like object:children-changed:remove events,
@@ -470,7 +470,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                        or ((event.type == "focus:") \
                            and (event.source.role == rolenames.ROLE_FRAME)):
 
-                        # If old ("factory") settings don't exist yet, save 
+                        # If old ("factory") settings don't exist yet, save
                         # a set, else restore the old application settings.
                         #
                         if not self._oldAppSettings:
@@ -491,6 +491,11 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                         #
                         appSettings = self.loadAppSettings(event.source.app)
 
+                        # Tell BrlTTY which commands we care about.
+                        #
+                        braille.setupKeyRanges(\
+                            orca_state.activeScript.brailleBindings.keys())
+
                     s = self._getScript(event.source.app)
                     s.processObjectEvent(event)
                     if retryCount:
@@ -827,6 +832,10 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         orca_state.activeScript = self._getScript(None)
 
+        # Tell BrlTTY which commands we care about.
+        #
+        braille.setupKeyRanges(orca_state.activeScript.brailleBindings.keys())
+
         self._registerEventListener("window:activate")
         self._registerEventListener("window:deactivate")
         self._registerEventListener("object:children-changed:remove")
