@@ -20,9 +20,9 @@
 """Manages the magnifier for orca.  [[[TODO: WDW - this is very very
 early in development.  One might even say it is pre-prototype.]]]"""
 
-__id__        = "$Id: mag.py,v 1.27 2006/06/10 00:54:47 wwalker Exp $"
-__version__   = "$Revision: 1.27 $"
-__date__      = "$Date: 2006/06/10 00:54:47 $"
+__id__        = "$Id: mag.py 1452 2006-09-19 21:30:55Z richb $"
+__version__   = "$Revision: 1452 $"
+__date__      = "$Date: 2006-09-19 17:30:55 -0400 (Tue, 19 Sep 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -97,6 +97,11 @@ _zoomer = None
 #
 _lastMouseEventTime = time.time()
 
+# If True, we're using gnome-mag >= 0.13.1 that allows us to control
+# where to draw the cursor and crosswires.
+#
+_pollMouseDisabled = False
+
 def __setROI(rect):
     """Sets the region of interest.
 
@@ -166,6 +171,11 @@ def __setROIPush(x, y):
         needNewROI = True
         newROI.y2 = y
         newROI.y1 = y - _roiHeight
+
+    # Well...we'll always update the ROI so the new gnome-mag API
+    # will redraw the crosswires for us.
+    #
+    #if needNewROI:
     if True:
         __setROI(newROI)
 
@@ -210,6 +220,12 @@ def __onMouseEvent(e):
 
     [x, y] = [e.detail1, e.detail2]
 
+    # If True, we're using gnome-mag >= 0.13.1 that allows us to
+    # control where to draw the cursor and crosswires.
+    #
+    if _pollMouseDisabled:
+        _zoomer.setPointerPos(x, y)
+
     if settings.magMouseTrackingMode == settings.MAG_MOUSE_TRACKING_MODE_PUSH:
         __setROIPush(x, y)
     elif settings.magMouseTrackingMode == settings.MAG_MOUSE_TRACKING_MODE_PROPORTIONAL:
@@ -258,6 +274,7 @@ def applySettings():
     global _minROIY
     global _maxROIX
     global _maxROIY
+    global _pollMouseDisabled
 
     ########################################################################
     #                                                                      #
@@ -267,6 +284,16 @@ def applySettings():
 
     _magnifier.clearAllZoomRegions()
 
+    try:
+        _magnifier.TargetDisplay = settings.magTargetDisplay
+    except:
+        pass
+
+    try:
+        _magnifier.SourceDisplay = settings.magSourceDisplay
+    except:
+        pass
+
     magnifierPBag = _magnifier.getProperties()
 
     try:
@@ -298,7 +325,7 @@ def applySettings():
             magnifierPBag, "cursor-size", 0)
 
     bonobo.pbclient_set_string(magnifierPBag, "cursor-set", "default")
-    
+
     # Convert the colorPreference string to something we can use.
     # The main issue here is that the color preferences are saved
     # as 4 byte values per color.  We only need 2 bytes, so we
@@ -365,6 +392,15 @@ def applySettings():
     zoomerPBag = _zoomer.getProperties()
     bonobo.pbclient_set_boolean(zoomerPBag, "is-managed", True)
 
+    # Try to use gnome-mag >= 0.13.1 to allow us to control where to
+    # draw the cursor and crosswires.
+    #
+    try:
+        bonobo.pbclient_set_boolean(zoomerPBag, "poll-mouse", False)
+        _pollMouseDisabled = True
+    except:
+        _pollMouseDisabled = False
+
     _zoomer.setMagFactor(settings.magZoomFactor, settings.magZoomFactor)
 
     bonobo.pbclient_set_boolean(
@@ -444,7 +480,6 @@ def magnifyAccessible(event, obj):
 
     if visibleX and visibleY:
         _zoomer.markDirty(_roi)
-        return
 
     # The algorithm is devised to move the ROI as little as possible, yet
     # favor the top left side of the object [[[TODO: WDW - the left/right
