@@ -24,19 +24,23 @@
 
 """Displays a GUI for the user to set Orca preferences."""
 
-__id__        = "$Id: orca_gui_prefs.py,v 1.58 2006/10/02 19:28:56 richb Exp $"
-__version__   = "$Revision: 1.58 $"
-__date__      = "$Date: 2006/10/02 19:28:56 $"
+__id__        = "$Id: orca_gui_prefs.py 1872 2007-01-03 17:33:35Z richb $"
+__version__   = "$Revision: 1872 $"
+__date__      = "$Date: 2007-01-03 12:33:35 -0500 (Wed, 03 Jan 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import os
+import commands
 import sys
 import debug
 import gettext
 import gtk
 import gtk.glade
+import gobject
+import pango   # for ellipsize property constants of CellRendererText
 import locale
+import atspi
 
 import acss
 import orca
@@ -45,6 +49,11 @@ import orca_prefs
 import orca_state
 import platform
 import settings
+import script
+import default    # for the default keyBindings
+import input_event
+import keybindings
+import braille
 import speech as speech
 import speechserver as speechserver
 
@@ -52,6 +61,9 @@ from orca_i18n import _  # for gettext support
 
 OS = None
 
+(HANDLER, DESCRIP, MOD_MASK1, MOD_USED1, KEY1, TEXT1, MOD_MASK2, MOD_USED2, KEY2, TEXT2, MODIF, EDITABLE) = range(12)
+
+
 class orcaSetupGUI(orca_glade.GladeWrapper):
 
     def _init(self):
@@ -71,6 +83,182 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         self.speechServersModel = self._initComboBox(self.speechServers)
         self.voicesModel        = self._initComboBox(self.voices)
 
+        # ***** Key Bindings treeview initialization *****
+
+        self.keyBindView = self.widgets.get_widget("keyBindingsTreeview")
+        self.keyBindingsModel = gtk.TreeStore(
+            gobject.TYPE_STRING,  # Handler name
+            gobject.TYPE_STRING,  # Human Readable Description
+            gobject.TYPE_STRING,  # Modifier mask 1
+            gobject.TYPE_STRING,  # Used Modifiers 1
+            gobject.TYPE_STRING,  # Modifier key name 1
+            gobject.TYPE_STRING,  # Text of the Key Binding Shown 1
+            gobject.TYPE_STRING,  # Modifier mask 2
+            gobject.TYPE_STRING,  # Used Modifiers 2
+            gobject.TYPE_STRING,  # Modifier key name 2
+            gobject.TYPE_STRING,  # Text of the Key Binding Shown 2
+            gobject.TYPE_BOOLEAN, # Key Modified by User
+            gobject.TYPE_BOOLEAN) # Row with fields editable or not
+        self.keyBindView.set_model(self.keyBindingsModel)
+        self.keyBindView.set_headers_visible(True)
+
+        # Treeview Column Implementation:
+        #      (HANDLER,    DESCRIP,
+        #       MOD_MASK1,  MOD_USED1,  KEY1,   TEXT1,
+        #       MOD_MASK2,  MOD_USED2,  KEY2,   TEXT2,
+        #       MODIF,      EDITABLE)
+
+        # HANDLER
+        #
+        column = gtk.TreeViewColumn(_("Handler"),
+                                    gtk.CellRendererText(),
+                                    text=HANDLER)
+        column.set_resizable(True)
+        column.set_visible(False)
+        column.set_sort_column_id(HANDLER)
+        self.keyBindView.append_column(column)
+
+        # DESCRIP
+        #
+        rendererText = gtk.CellRendererText()
+        rendererText.set_property("ellipsize", pango.ELLIPSIZE_END)
+        column = gtk.TreeViewColumn(_("Function"), rendererText, text=DESCRIP)
+        column.set_resizable(True)
+        column.set_min_width(380)
+        column.set_sort_column_id(DESCRIP)
+        self.keyBindView.append_column(column)
+
+        # MOD_MASK1
+        #
+        column = gtk.TreeViewColumn(_("Mod.Mask 1"),
+                                    gtk.CellRendererText(),
+                                    text=MOD_MASK1)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(MOD_MASK1)
+        self.keyBindView.append_column(column)
+
+        # MOD_USED1
+        #
+        column = gtk.TreeViewColumn(_("Use Mod.1"),
+                                    gtk.CellRendererText(),
+                                    text=MOD_USED1)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(MOD_USED1)
+        self.keyBindView.append_column(column)
+
+        # KEY1
+        #
+        column = gtk.TreeViewColumn(_("Key1"),
+                                    gtk.CellRendererText(),
+                                    text=KEY1)
+        column.set_resizable(True)
+        column.set_visible(False)
+        column.set_sort_column_id(KEY1)
+        self.keyBindView.append_column(column)
+
+        # TEXT1
+        #
+        rendererText = gtk.CellRendererText()
+        rendererText.connect("editing-started",
+                             self.editingKey,
+                             self.keyBindingsModel)
+        rendererText.connect('edited',
+                             self.editedKey,
+                             self.keyBindingsModel,
+                             MOD_MASK1, MOD_USED1, KEY1, TEXT1)
+        column = gtk.TreeViewColumn(_("Key Binding"),
+                                    rendererText,
+                                    text=TEXT1,
+                                    editable=EDITABLE)
+        column.set_resizable(True)
+        column.set_sort_column_id(TEXT1)
+        self.keyBindView.append_column(column)
+
+        # MOD_MASK2
+        #
+        column = gtk.TreeViewColumn(_("Mod.Mask 2"),
+                                    gtk.CellRendererText(),
+                                    text=MOD_MASK2)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(MOD_MASK2)
+        self.keyBindView.append_column(column)
+
+        # MOD_USED2
+        #
+        column = gtk.TreeViewColumn(_("Use Mod.2"),
+                                    gtk.CellRendererText(),
+                                    text=MOD_USED2)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(MOD_USED2)
+        self.keyBindView.append_column(column)
+
+        # KEY2
+        #
+        column = gtk.TreeViewColumn(_("Key2"), rendererText, text=KEY2)
+        column.set_resizable(True)
+        column.set_visible(False)
+        column.set_sort_column_id(KEY2)
+        self.keyBindView.append_column(column)
+
+        # TEXT2
+        #
+        rendererText = gtk.CellRendererText()
+        rendererText.connect("editing-started",
+                             self.editingKey,
+                             self.keyBindingsModel)
+        rendererText.connect('edited',
+                             self.editedKey,
+                             self.keyBindingsModel,
+                             MOD_MASK2, MOD_USED2, KEY2, TEXT2)
+        column = gtk.TreeViewColumn(_("Alternate"),
+                                    rendererText,
+                                    text=TEXT2,
+                                    editable=EDITABLE)
+        column.set_resizable(True)
+        column.set_sort_column_id(TEXT2)
+        self.keyBindView.append_column(column)
+
+        # MODIF
+        #
+        rendererToggle = gtk.CellRendererToggle()
+        rendererToggle.connect('toggled',
+                               self.keyModifiedToggle,
+                               self.keyBindingsModel,
+                               MODIF)
+        column = gtk.TreeViewColumn(_("Modified"),
+                                    rendererToggle,
+                                    active=MODIF,
+                                    activatable=EDITABLE)
+        #column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(MODIF)
+        self.keyBindView.append_column(column)
+
+        # EDITABLE
+        #
+        rendererToggle = gtk.CellRendererToggle()
+        rendererToggle.set_property('activatable',False)
+        column = gtk.TreeViewColumn(_("Modified"),
+                                    rendererToggle,
+                                    active=EDITABLE)
+        column.set_visible(False)
+        column.set_resizable(True)
+        column.set_sort_column_id(EDITABLE)
+        self.keyBindView.append_column(column)
+
+        # Populates the treeview with all the keybindings:
+        #
+        self._populateKeyBindings()
+
+        self.keyBindView.show()
+
+        self.window = self.widgets.get_widget("orcaSetupWindow")
+        self.window.resize(790,580);
+
         self._setKeyEchoItems()
 
         factories = speech.getSpeechServerFactories()
@@ -204,6 +392,9 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         self.speechIndentationCheckbutton.set_active(\
             prefs["enableSpeechIndentation"])
 
+        self.speakBlankLinesCheckButton.set_active(\
+            prefs["speakBlankLines"])
+
         # Braille pane.
         #
         self.brailleSupportCheckbutton.set_active(prefs["enableBraille"])
@@ -370,6 +561,18 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         #
         self.showMainWindowCheckButton.set_active(prefs["showMainWindow"])
 
+        self.disableKeyGrabPref = commands.getoutput( \
+                              "gconftool-2 --get /apps/gksu/disable-grab")
+        if self.disableKeyGrabPref == "true":
+            self.disableKeyGrabCheckButton.set_active(True)
+        else:
+            self.disableKeyGrabCheckButton.set_active(False)
+
+        if prefs["keyboardLayout"] == settings.GENERAL_KEYBOARD_LAYOUT_DESKTOP:
+            self.generalDesktopButton.set_active(True)
+        else:
+            self.generalLaptopButton.set_active(True)
+
     def _getComboBoxIndex(self, combobox, str):
         """ For each of the entries in the given combo box, look for str.
             Return the index of the entry if str is found.
@@ -749,6 +952,197 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
 
         speech.speak(text)
 
+    def _createNode(self, appName):
+        """Create a new root node in the TreeStore model with the name of the
+            application.
+
+        Arguments:
+        - appName: the name of the TreeStore Node (the same of the application)
+        """
+
+        model = self.keyBindingsModel
+
+        iter = model.append(None)
+        model.set(iter,
+                  DESCRIP, appName,
+                  MODIF, False)
+
+        return iter
+
+    def _getIterOf(self, appName):
+        """Returns the gtk.TreeIter of the TreeStore model
+        that matches the application name passed as argument
+
+        Arguments:
+        - appName: a string with the name of the application of the node wanted
+                    it's the same that the field DESCRIP of the model treeStore
+        """
+
+        model = self.keyBindingsModel
+
+        for row in model:
+            if ((model.iter_depth(row.iter) == 0) \
+                and (row[DESCRIP] == appName)):
+                return row.iter
+
+        return None
+
+    def _addAlternateKeyBinding(self, kb):
+        """Adds an alternate keybinding to the existing handler and
+        returns true.  In case it doesn't exist yet, just returns
+        false.
+
+        Argument:
+        - kb: the keybinding to be added as an alternate keybinding.
+        """
+
+        model = self.keyBindingsModel
+        iter = model.get_iter_first()
+        exist = False
+
+        while iter != None:
+            iterChild = model.iter_children(iter)
+            while iterChild != None:
+                if model.get(iterChild,DESCRIP)[0] == kb.handler._description:
+                    exist = True
+                    model.set(iterChild,
+                              MOD_MASK2, kb.modifier_mask,
+                              MOD_USED2, kb.modifiers,
+                              KEY2, kb.keysymstring,
+                              TEXT2,keybindings.getModifierNames(kb.modifiers)\
+                                  + kb.keysymstring)
+                iterChild = model.iter_next(iterChild)
+            iter = model.iter_next(iter)
+
+        return exist
+
+    def _insertRow(self, handl, kb, parent=None, modif=False):
+        """Appends a new row with the new keybinding data to the treeview
+
+        Arguments:
+        - handl:  the name of the handler associated to the keyBinding
+        - kb:     the new keybinding.
+        - parent: the parent node of the treeview, where to append the kb
+        - modif:  whether to check the modified field or not.
+
+        Returns a gtk.TreeIter pointing at the new row.
+        """
+
+        model = self.keyBindingsModel
+
+        if parent == None:
+            parent = self._getIterOf(_("Orca"))
+
+        if parent != None:
+            myiter = model.append(parent)
+            model.set (myiter,
+                       HANDLER,   handl,
+                       DESCRIP,   kb.handler._description,
+                       MOD_MASK1, kb.modifier_mask,
+                       MOD_USED1, kb.modifiers,
+                       KEY1,      kb.keysymstring,
+                       TEXT1,     keybindings.getModifierNames(kb.modifiers) \
+                                  + kb.keysymstring,
+                       MODIF,     modif,
+                       EDITABLE,  True)
+            return myiter
+        else:
+            return None
+
+    def _insertRowBraille(self, handl, com, inputEvHand, parent=None, modif=False):
+        """Appends a new row with the new braille binding data to the treeview
+
+        Arguments:
+        - handl:       the name of the handler associated to the brailleBinding
+        - com:         the BrlTTY command
+        - inputEvHand: the inputEventHandler with the new brailleBinding
+        - parent:      the parent node of the treeview, where to append the kb
+        - modif:       whether to check the modified field or not.
+
+        Returns a gtk.TreeIter pointing at the new row.
+        """
+
+        model = self.keyBindingsModel
+
+        if parent == None:
+            parent = self._getIterOf(_("Braille Bindings"))
+
+        if parent != None:
+            myiter = model.append(parent)
+            model.set (myiter,
+                       HANDLER,  handl,
+                       DESCRIP,  inputEvHand._description,
+                       KEY1,     com,
+                       TEXT1,    braille.command_name[com],
+                       MODIF,    modif,
+                       EDITABLE, False)
+            return myiter
+        else:
+            return None
+
+    def _markModified(self):
+        """ Mark as modified the user custom key bindings:
+        """
+
+        try:
+            defScript = default.Script(None)
+            defScript.setupInputEventHandlers()
+            keyBinds = keybindings.KeyBindings()
+            keyBinds = settings.overrideKeyBindings(defScript,keyBinds)
+            keyBind = keybindings.KeyBinding(None,None,None,None)
+            treeModel = self.keyBindingsModel
+
+            iter = treeModel.get_iter_first()
+            while iter != None:
+                iterChild = treeModel.iter_children(iter)
+                while iterChild != None:
+                    descrip = treeModel.get_value(iterChild, DESCRIP)
+                    keyBind.handler=input_event.InputEventHandler(None,descrip)
+                    if keyBinds.hasKeyBinding(keyBind,
+                                              typeOfSearch="description"):
+                        treeModel.set_value(iterChild, MODIF, True)
+                    iterChild = treeModel.iter_next(iterChild)
+                iter = treeModel.iter_next(iter)
+        except:
+            debug.printException(debug.LEVEL_SEVERE)
+
+    def _populateKeyBindings(self):
+        """Fills the TreeView with the list of Orca keybindings
+        """
+        self.keyBindingsModel.clear()
+
+        self.kbindings = keybindings.KeyBindings()
+
+        # KeyBindings from the default script, in default.py (Orca's default)
+        #
+        iterOrca = self._createNode(_("Orca"))
+        defScript = default.Script(None)
+        self.kbindingsDef = defScript.getKeyBindings()
+
+        for kb in self.kbindingsDef.keyBindings:
+            if not self.kbindings.hasKeyBinding(kb, typeOfSearch="strict"):
+                if not self._addAlternateKeyBinding(kb):
+                    handl = defScript.getInputEventHandlerKey(kb.handler)
+                    self._insertRow(handl, kb, iterOrca)
+
+        self.orcaModKeyEntry = self.widgets.get_widget("orcaModKeyEntry")
+        self.orcaModKeyEntry.set_text(
+            str(settings.orcaModifierKeys)[1:-1].replace("'",""))
+
+        self._markModified()
+
+        # Braille Bindings from default script, in default.py
+        #
+        iterBB = self._createNode(_("Braille Bindings"))
+
+        self.bbindings = defScript.getBrailleBindings()
+        for com, inputEvHand in self.bbindings.iteritems():
+            handl = defScript.getInputEventHandlerKey(inputEvHand)
+            self._insertRowBraille(handl, com, inputEvHand, iterBB)
+
+        self.keyBindView.expand_all()
+        self.keyBindingsModel.set_sort_column_id(TEXT1, gtk.SORT_ASCENDING)
+
     def speechSupportChecked(self, widget):
         """Signal handler for the "toggled" signal for the
            speechSupportCheckbutton GtkCheckButton widget. The user has
@@ -834,6 +1228,18 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
         enable = widget.get_active()
         self.prefsDict["enableSpeechIndentation"] = enable
 
+    def speakBlankLinesChecked(self, widget):
+        """Signal handler for the "toggled" signal for the
+           speakBlankLinesCheckButton GtkCheckButton widget. The user has
+           [un]checked the 'Speak blank lines' checkbox.
+           Set the 'speakBlankLines' preference to the new value.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        self.prefsDict["speakBlankLines"] = widget.get_active()
+
     def brailleSupportChecked(self, widget):
         """Signal handler for the "toggled" signal for the
            brailleSupportCheckbutton GtkCheckButton widget. The user has
@@ -1335,6 +1741,47 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
 
         self.prefsDict["enableMagZoomerColorInversion"] = widget.get_active()
 
+    def keyModifiedToggle(self, cell, path, model, col):
+        """When the user changes a checkbox field (boolean field)"""
+
+        model[path][col] = not model[path][col]
+        return
+
+    def editingKey(self, cell, editable, path, treeModel):
+        """Starts user input of a Key for a selected key binding"""
+
+        editable.set_text(_("enter new key"))
+        orca_state.capturingKeys = True
+        return
+
+    def editedKey(self, cell, path, new_text, treeModel, modMask, modUsed, key, text):
+        """The user changes the key for a Keybinding:
+            update the model of the treeview
+        """
+
+        newKeyEvent = orca_state.lastCapturedKey
+        newKey = str(orca_state.lastCapturedKey.event_string)
+        mods = keybindings.getModifierNames(newKeyEvent.modifiers)
+
+        orca_state.capturingKeys = False
+
+        if (newKey != treeModel[path][key]) \
+           or (newKeyEvent.modifiers != int(treeModel[path][modUsed])):
+            iter = treeModel.get_iter_from_string(path)
+            treeModel.set(iter,
+                          modMask, newKeyEvent.modifiers,
+                          modUsed, newKeyEvent.modifiers,
+                          key, newKey,
+                          text, mods + newKey,
+                          MODIF, True)
+            speech.stop()
+            speech.speak(_("The new key is: %s") % newKey)
+        else:
+            speech.stop()
+            speech.speak(_("The key entered is the same. Nothing changed."))
+
+        return
+
     def magSourceDisplayChanged(self, widget):
         """Signal handler for the "changed" signal for the
            magSourceDisplayDisplayEntry GtkEntry widget.
@@ -1374,6 +1821,44 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
 
         self.prefsDict["showMainWindow"] = widget.get_active()
 
+    def disableKeyGrabChecked(self, widget):
+        """Signal handler for the "toggled" signal for the
+           disableKeyGrabCheckButton GtkCheckButton widget.
+           The user has [un]checked the 'Disable gksu keyboard grab'
+           checkbox. Set the gconf '/apps/gksu/disable-grab' resource
+           to the new value.
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        self.disableKeyGrabPref = widget.get_active()
+
+    def keyboardLayoutChanged(self, widget):
+        """Signal handler for the "toggled" signal for the generalDesktopButton,
+           or generalLaptopButton GtkRadioButton widgets. The user has
+           toggled the keyboard layout value. If this signal was
+           generated as the result of a radio button getting selected
+           (as opposed to a radio button losing the selection), set the
+           'keyboardLayout' preference to the new value. Also set the
+           matching list of Orca modifier keys
+
+        Arguments:
+        - widget: the component that generated the signal.
+        """
+
+        if widget.get_active():
+            if widget.get_label() == _("_Desktop"):
+                self.prefsDict["keyboardLayout"] = \
+                    settings.GENERAL_KEYBOARD_LAYOUT_DESKTOP
+                self.prefsDict["orcaModifierKeys"] = \
+                    settings.DESKTOP_MODIFIER_KEYS
+            else:
+                self.prefsDict["keyboardLayout"] = \
+                    settings.GENERAL_KEYBOARD_LAYOUT_LAPTOP
+                self.prefsDict["orcaModifierKeys"] = \
+                    settings.LAPTOP_MODIFIER_KEYS
+
     def helpButtonClicked(self, widget):
         """Signal handler for the "clicked" signal for the helpButton
            GtkButton widget. The user has clicked the Help button.
@@ -1391,7 +1876,7 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
            previously been enabled, warn the user that they will need to
            log out. Shut down any active speech servers that were started.
            Reload the users preferences to get the new speech, braille and
-           key echo value to take effect. Do not dismiss the configuration 
+           key echo value to take effect. Do not dismiss the configuration
            window.
 
         Arguments:
@@ -1408,24 +1893,32 @@ class orcaSetupGUI(orca_glade.GladeWrapper):
             settings.HYPERLINK_VOICE : self.hyperlinkVoice
         }
 
-        if orca_prefs.writePreferences(self.prefsDict):
+        if self.disableKeyGrabPref:
+            keyGrabState = "true"
+        else:
+            keyGrabState = "false"
+        os.system("gconftool-2 --type bool --set " \
+                  + "/apps/gksu/disable-grab " + keyGrabState)
+
+        if orca_prefs.writePreferences(self.prefsDict, self.keyBindingsModel):
             self._say(_("Accessibility support for GNOME has just been enabled."))
             self._say(_("You need to log out and log back in for the change to take effect."))
 
         for factory in self.workingFactories:
             factory.SpeechServer.shutdownActiveServers()
         orca.loadUserSettings()
+        self._populateKeyBindings()
 
     def cancelButtonClicked(self, widget):
         """Signal handler for the "clicked" signal for the cancelButton
            GtkButton widget. The user has clicked the Cancel button.
-           Don't write out the preferences. Hide the configuration window.
+           Don't write out the preferences. Destroy the configuration window.
 
         Arguments:
         - widget: the component that generated the signal.
         """
 
-        self.orcaSetupWindow.hide()
+        self.orcaSetupWindow.destroy()
 
     def okButtonClicked(self, widget):
         """Signal handler for the "clicked" signal for the okButton
