@@ -21,9 +21,9 @@
 of manipulating text - both the user and the system put text into the
 system and we try to determine which was which and why."""
 
-__id__        = "$Id: gnome-terminal.py,v 1.15 2006/10/05 16:31:04 wwalker Exp $"
-__version__   = "$Revision: 1.15 $"
-__date__      = "$Date: 2006/10/05 16:31:04 $"
+__id__        = "$Id: gnome-terminal.py 1584 2006-10-19 18:16:54Z richb $"
+__version__   = "$Revision: 1584 $"
+__date__      = "$Date: 2006-10-19 14:16:54 -0400 (Thu, 19 Oct 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -54,6 +54,11 @@ class Script(default.Script):
 
         default.Script.__init__(self, app)
 
+        # By default, don't present if gnome-terminal is not the active 
+        # application.
+        #
+        self.presentIfInactive = False
+
     #def onWindowActivated(self, event):
     #    # Sets the context to the top level window first, so we can
     #    # get information about it the window we just moved to.
