@@ -21,14 +21,20 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py,v 1.68 2006/07/19 20:46:28 richb Exp $"
-__version__   = "$Revision: 1.68 $"
-__date__      = "$Date: 2006/07/19 20:46:28 $"
+__id__        = "$Id: settings.py 1818 2006-12-13 01:21:31Z wwalker $"
+__version__   = "$Revision: 1818 $"
+__date__      = "$Date: 2006-12-12 20:21:31 -0500 (Tue, 12 Dec 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import os
 import re
 import sys
+
+screenWidth = 640
+screenHeight = 480
+tty = 7
+
 try:
     # This can fail due to gtk not being available.  We want to
     # be able to recover from that if possible.  The main driver
@@ -36,13 +42,22 @@ try:
     # the desktop is not running.
     #
     import gtk.gdk
-    screenWidth = gtk.gdk.screen_get_default().get_width()
-    screenHeight = gtk.gdk.screen_get_default().get_height()
-except:
-    # Just default to something that hopefully everybody has.
+    _display = gtk.gdk.display_get_default()
+    _screen = _display.get_default_screen()
+    _root_window = _screen.get_root_window()
+
+    # These are used for placing the magnifier zoomer.
     #
-    screenWidth = 640
-    screenHeight = 480
+    screenWidth = _screen.get_width()
+    screenHeight = _screen.get_height()
+
+    # We want to know what the tty is so we can send it to BrlAPI
+    # if possible.
+    #
+    (atom, format, data) = _root_window.property_get("XFree86_VT")
+    tty = data[0]
+except:
+    pass
 
 import debug
 from acss import ACSS
@@ -87,9 +102,20 @@ userCustomizableSettings = [
     "enableMagZoomerColorInversion",
     "magSmoothingMode",
     "magMouseTrackingMode",
+    "magSourceDisplay",
+    "magTargetDisplay",
     "verbalizePunctuationStyle",
+    "showMainWindow",
+    "keyboardLayout",
+    "speakBlankLines",
 ]
 
+# The name of the module that hold the user interface for the main window
+# for Orca. This module is expected to have two methods, showMainUI and
+# hideMainUI, which will show and hide the main window GUI.
+#
+mainWindowModule = "orca_gui_main"
+
 # The name of the modules that hold the user interface for setting
 # Orca preferences.  Each module is expected to have the method,
 # showPreferencesUI, which will prompt the user for preferences.
@@ -97,6 +123,17 @@ userCustomizableSettings = [
 guiPreferencesModule    = "orca_gui_prefs"
 consolePreferencesModule= "orca_console_prefs"
 
+# The name of the module that hold the user interface for quitting Orca.
+# This module is expected to have the method, showQuitUI, which will
+# display the quit GUI.
+#
+quitModule = "orca_quit"
+
+# The name of the module that holds the user interface for performing a
+# flat review find.
+#
+findModule = "orca_gui_find"
+
 # A list of keys that can serve as the Orca modifier key.  The list is
 # so we can provide better cross platform support (e.g., Sun keyboard
 # vs. PC-104 keyboard layouts).  When any of these keys is pressed,
@@ -104,7 +141,17 @@ consolePreferencesModule= "orca_console_prefs"
 # a KeyboardEvent input event.  The keys are currently compared to the
 # event_string of a keyboard input event from AT-SPI.
 #
-orcaModifierKeys        = ["Insert", "KP_Insert"]
+# The initial set of modifier keys is dependant upon whether the user
+# has specified a desktop or a laptop keyboard layout.
+#
+DESKTOP_MODIFIER_KEYS = ["Insert", "KP_Insert"]
+LAPTOP_MODIFIER_KEYS  = ["Caps_Lock"]
+orcaModifierKeys      = DESKTOP_MODIFIER_KEYS
+
+# A new modifier to use (set by the press of any key in the
+# orcaModifierKeys list) to represent the Orca modifier.
+#
+MODIFIER_ORCA = 8
 
 # Verbosity levels (see setBrailleVerbosityLevel and
 # setSpeechVerbosityLevel).  These will have an impact on the various
@@ -167,7 +214,7 @@ HYPERLINK_VOICE         = "hyperlink"
 voices = {
     DEFAULT_VOICE   : ACSS({}),
     UPPERCASE_VOICE : ACSS({ACSS.AVERAGE_PITCH : 6}),
-    HYPERLINK_VOICE : ACSS({ACSS.AVERAGE_PITCH : 8})
+    HYPERLINK_VOICE : ACSS({})
 }
 
 # If True, enable speaking of speech indentation and justification.
@@ -249,6 +296,14 @@ MAG_MOUSE_TRACKING_MODE_PROPORTIONAL = 2
 MAG_MOUSE_TRACKING_MODE_PUSH         = 3
 magMouseTrackingMode                 = MAG_MOUSE_TRACKING_MODE_CENTERED
 
+# Magnification source display
+#
+magSourceDisplay                 = ''
+
+# Magnification target display
+#
+magTargetDisplay                 = ''
+
 # if True, enable word echo.
 # Note that it is allowable for both enableEchoByWord and enableKeyEcho
 # to be True
@@ -281,12 +336,44 @@ enableFunctionKeys      = True
 #
 enableActionKeys        = True
 
+# If True, show the main Orca window.
+#
+showMainWindow          = True
+
+# Keyboard layout options (see keyboardLayout).
+#
+GENERAL_KEYBOARD_LAYOUT_DESKTOP = 1
+GENERAL_KEYBOARD_LAYOUT_LAPTOP  = 2
+keyboardLayout                  = GENERAL_KEYBOARD_LAYOUT_DESKTOP
+
+# If True, speak blank lines.
+#
+speakBlankLines         = True
 
 # If True, reads all the table cells in the current row rather than just
 # the current one.
 #
 readTableCellRow    = True
 
+# The default set of text attributes to speak to the user. Specific
+# application scripts (or individual users can override these values if
+# so desired. Each of these text attributes is of the form <key>:<value>;
+# The <value> part will be the "default" value for that attribute. In
+# other words, if the attribute for a given piece of text has that value,
+# it won't be spoken. If no value part is given, then that attribute will
+# always be spoken.
+
+enabledTextAttributes = "size:; family-name:; weight:400; indent:0; underline:none; strikethrough:false; justification:left; style:normal;"
+
+# The limit to enable a repeat character count to be spoken.
+# If set to 0, then there will be no repeat character count.
+# Each character will be spoken singularly (i.e. "dash dash
+# dash dash dash" instead of "five dash characters").
+# If the value is set to 1, 2 or 3 then it's treated as if it was
+# zero. In other words, no repeat character count is given.
+#
+repeatCharacterLimit = 4
+
 # Script developer feature.  If False, just the default script
 # will be used.  Helps determine difference between custom
 # scripts and the default script behavior.
@@ -319,6 +406,25 @@ cacheAccessibles        = True
 #
 learnModeEnabled        = False
 
+# The location of the user's preferences. By default this is ~/.orca.
+# It can be overridden by the Orca -d command line option.
+#
+userPrefsDir = os.path.join(os.environ["HOME"], ".orca")
+
+# If non-zero, we use time.sleep() in various places to attempt to
+# free up the global interpreter lock.  Take a look at the following
+# URLs for more information:
+#
+# http://mail.python.org/pipermail/python-list/2002-October/126632.html
+# http://twistedmatrix.com/pipermail/twisted-python/2005-July/011052.html
+# http://www.pyzine.com/Issue001/Section_Articles/article_ThreadingGlobalInterpreter.html
+#
+gilSleepTime            = 0.00001
+
+# If True, use the gidle __blockPreventor() code in atspi.py.
+#
+useBlockPreventor       = False
+
 # If True, we use the bonobo main loop provided by bonobo to handle
 # all events in atspi.py.  If False, we create our own loop.
 #
@@ -330,6 +436,15 @@ useBonoboMain           = True
 #
 debugEventQueue         = False
 
+# The timeout value (in seconds) and callback used to determine if
+# Orca has hung or not.  The only setting one should muck with here is
+# the timeoutTime unless you want to create a custom callback handler
+# for the timeout.  See braille.py, atspi.py, and orca.py:init for how
+# these are used.
+#
+timeoutTime             = 10   # a value of 0 means don't do hang checking
+timeoutCallback         = None # Set by orca.py:init to orca.timeout
+
 # Assists with dealing with CORBA COMM_FAILURES.  A failure doesn't
 # always mean an object disappeared - there just might be a network
 # glitch.  So, on COMM_FAILURES, we might retry a few times before
@@ -345,6 +460,19 @@ commFailureAttemptLimit = 5
 #
 doubleClickTimeout = 0.5
 
+# Allow for the customization of key bindings.
+#
+def overrideKeyBindings(script, keyBindings):
+    return keyBindings
+
+# Which packages to search, and the order in which to search,
+# for application settings.  These packages are expected to be on
+# the PYTHONPATH and/or subpackages of the "orca" package.
+# REMEMBER: to make something a package, the directory has to
+# have a __init__.py file in it.
+#
+settingsPackages          = ["app-settings"]
+
 # Which packages to search, and the order in which to search,
 # for custom scripts.  These packages are expected to be on
 # the PYTHONPATH and/or subpackages of the "orca" package.
@@ -412,4 +540,9 @@ setScriptMapping(re.compile(_('[\S\s]*StarOffice[\s\S]*')), "StarOffice")
 setScriptMapping(re.compile(_('soffice.bin')), "StarOffice")
 setScriptMapping(re.compile(_('[Ee]volution')), "Evolution")
 setScriptMapping(re.compile(_('Deer Park')), "Mozilla")
+setScriptMapping(re.compile(_('yelp')), "Mozilla")
 setScriptMapping(re.compile(_('Bon Echo')), "Mozilla")
+setScriptMapping(re.compile(_('Minefield')), "Mozilla")
+setScriptMapping(re.compile(_('bug-buddy')), "gnome_segv2")
+setScriptMapping(re.compile(_('vte')), "gnome-terminal")
+setScriptMapping(re.compile(_('time-admin')), "users-admin")
