@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py,v 1.88 2006/12/12 21:07:08 richb Exp $"
-__version__   = "$Revision: 1.88 $"
-__date__      = "$Date: 2006/12/12 21:07:08 $"
+__id__        = "$Id: speechgenerator.py 1847 2006-12-19 21:23:59Z wwalker $"
+__version__   = "$Revision: 1847 $"
+__date__      = "$Date: 2006-12-19 16:23:59 -0500 (Tue, 19 Dec 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1259,7 +1259,7 @@ class SpeechGenerator:
         utterances = []
 
         if (not already_focused):
-            if settings.readTableCellRow:
+            if settings.readTableCellRow and obj.parent.table:
                 parent = obj.parent
                 row = parent.table.getRowAtIndex(obj.index)
                 column = parent.table.getColumnAtIndex(obj.index)
