@@ -19,9 +19,9 @@
 
 """Provides various utility functions for Orca."""
 
-__id__        = "$Id: util.py,v 1.53 2006/11/30 03:02:23 lmonsanto Exp $"
-__version__   = "$Revision: 1.53 $"
-__date__      = "$Date: 2006/11/30 03:02:23 $"
+__id__        = "$Id: util.py 1885 2007-01-05 16:38:30Z richb $"
+__version__   = "$Revision: 1885 $"
+__date__      = "$Date: 2007-01-05 11:38:30 -0500 (Fri, 05 Jan 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -601,6 +601,8 @@ def _addRepeatSegment(segment, line, respectPunctuation=True):
            or (isPunctChar and (style <= level)):
             repeatChar = chnames.getCharacterName(segment[0])
             line += _(" %d %s characters ") % (count, repeatChar)
+        else:
+            line += segment
     else:
         line += segment
 
@@ -775,6 +777,15 @@ def getTopLevel(obj):
 
     return obj
 
+def getTopLevelName(obj):
+    """ Returns the name of the top-level object. See getTopLevel.
+    """
+    top = getTopLevel(obj)
+    if (not top) or (not top.name):
+        return ""
+    else:
+        return top.name
+
 def getTextLineAtCaret(obj):
     """Gets the line of text where the caret is.
 
@@ -792,10 +803,15 @@ def getTextLineAtCaret(obj):
     if not text:
         return ["", 0, 0]
 
+    if text.caretOffset == text.characterCount:
+        caretOffset = max(0, text.caretOffset - 1)
+    else:
+        caretOffset = text.caretOffset
+
     # Get the line containing the caret
     #
     [string, startOffset, endOffset] = text.getTextAtOffset(
-        text.caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+        caretOffset, atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
 
     # Sometimes we get the trailing line-feed-- remove it
     #
